/******************************************************************/
/* filename :       vf_irfcs_qgrid.g                              */
/* Description:                                                   */
/*                                                                */
/******************************************************************/

new;
closeall; 
cls;

#include vf_spec.g; 
#include pathspec.g;

// Define some constants 
th_dim         = 1; 
q_dim          = nvar_;
critval_nsim_  = 5000;
lambda_nrep_   = 2000;
cs_step_scale_ = 1/100;
var_min        = 1;
var_max        = 4;
h_min          = 1;
h_max          = nirf_; 
sel_str        =  "vl" $+ ftos(var_min,"%*.*lf",1,0)
               $+ "vh" $+ ftos(var_max,"%*.*lf",1,0)
               $+ "hl" $+ ftos(h_min,"%*.*lf",1,0)
               $+ "hm" $+ ftos(h_max,"%*.*lf",1,0);

start_bayes = 0;
freqrun = "2"; 

/* import q_grid */
qgridpath = workpath $+ "\\" $+ prioname;
load path = ^qgridpath q_grid;

/******************************************************** 
* Import Bands from Preliminary Analysis
********************************************************/

if start_bayes == 1;
   lvarirf = workpath $+ "\\..\\PureSignBayes\\vb_d1t1p3\\results\\mhrun1\\varirfs";
   open fhvarirf = ^lvarirf for read;
   varirf_m = readr(fhvarirf,1);
   varirf_s = readr(fhvarirf,1);
   varirf_l = readr(fhvarirf,1);
   varirf_h = readr(fhvarirf,1);
   closeall;
   freqrun = freqrun $+ "b"; 
else;
   lvarirf = workpath $+ "\\..\\PureSignFreqBonfDiagW\\vf_d1t1p2\\results\\freqrun2\\varirfs";
   open fhvarirf = ^lvarirf for read;
   varirf_l = readr(fhvarirf,1);
   varirf_h = readr(fhvarirf,1);
   closeall;
   freqrun = freqrun $+ "f"; 
endif;

varirf_l = reshape(varirf_l,nvar_,nirf_)';
varirf_h = reshape(varirf_h,nvar_,nirf_)';

/******************************************************** 
* Import data series                                    *
********************************************************/

#include vf_loaddata.g;
k = 1+nvar_*nlags_;

/******************************************************** 
* Iterate over variables/horizons                       *
********************************************************/

/** start log file **/
outstr = resupath $+ "\\" $+ ftos(timedt,"%*.*lf",1,0) $+ "freqirf" $+ freqrun $+ sel_str $+ ".log"; 
output file = ^outstr reset;

varirf_freq_l = varirf_l;
varirf_freq_h = varirf_h;

varindx = var_min;
do until varindx > var_max;

   horizonindx = h_min;
   do until horizonindx > h_max;
   
      /** set start time **/
      tstart = date;   

      /* estimate impulse responses
      */
      { YYirf, phi_hat, sigma_hat } = estimate_irfs(YY,XX,nirf_);
      { YYirfselect, YYirfrestr }   = select_irfs(YYirf,varindx,horizonindx,signrestrictionsindx_);
      psi_hat_   = vec(YYirfselect');
      YYirfrestr_= YYirfrestr;
      mu_dim_    = rows(YYirfrestr)-th_dim;

      /* estimate lambda (covariance matrix of phi_hat)
      ** we are using a fixed seed for the bootstrap of phi_hat
      */
      "Computing Lambda_hat";
      pause(1e-5);
      lambda_hat_ = estimate_lambda(YY,XX,lambda_nrep_,varindx,horizonindx,signrestrictionsindx_);
      lambda_hat_ = lambda_hat_;
      "";

      /* width of preliminary interval
      */
      hpd_width = varirf_h[horizonindx,varindx] -
                  varirf_l[horizonindx,varindx] ;

      cs_step = hpd_width*cs_step_scale_;
      
      /***********************************************************************
      ** search for upper bound of HPD interval
      ** Initialization: 
      ** if starting value is in CS, then we need to increase upper bound; 
      **    direction = 1
      ** if starting value is NOT in CS, then we need to decrease upper bound; 
      **    direction = -1;
      */
      "Computing Upper Bound";
      pause(1e-5);   

      cs_freq_h   = varirf_freq_h[horizonindx,varindx];      
      th_ = cs_freq_h;
      
      {GT_lesscritval, q_grid, n_nonbinding} = computeGT(q_grid);

      if GT_lesscritval == 1;
         /* starting point is included in CS */
         direction = 1; /* increase upper bound */
      else;
         /* starting point is not included in CS */
         direction = -1; /* decrease upper bound */
      endif;
            
      terminate = 0;

      do until terminate;
         
         th_ = cs_freq_h;

         {GT_lesscritval, q_grid, n_nonbinding} = computeGT(q_grid);

         /* Termination criterion
         */
         if direction == 1;
            /* we are raising the upper bound and find transition
            ** included -> not included OR upper bound IRF <= 0
            */   
            if (GT_lesscritval == 0)
                or ( (YYirfrestr[1]==-1) and ((cs_freq_h+cs_step) > 0) );
                terminate = 1;
            endif;
         else; 
            /* we are decreasing the upper bound and trying to find transition
            ** not included -> included without violating the constraint IRF <=0
            */
            if (GT_lesscritval == 1) and (NOT ((YYirfrestr[1]==-1) and (cs_freq_h > 0)));
               terminate = 1;
            endif;
         endif;
                           
         cs_freq_h = cs_freq_h + direction*cs_step;

      endo;
       
      if direction == 1;
         /* we increased the upper bound too much */ 
         varirf_freq_h[horizonindx,varindx] = cs_freq_h - cs_step;
         cs_freq_h = cs_freq_h - cs_step;
      else;
         /* we lowered the upper bound */
         varirf_freq_h[horizonindx,varindx] = cs_freq_h;
      endif;
      "Upper Bound" cs_freq_h;
      pause(1e-2);

      /***********************************************************************
      ** search for lower bound of CONFIDENCE interval
      ** Initialization: 
      ** if starting value is in CS, then we need to descrease lower bound; 
      **    direction = -1
      ** if starting value is NOT in CS, then we need to increase upper bound; 
      **    direction = 1;
      */
      "Computing Lower Bound";
      pause(1e-5);
     
      cs_freq_l = varirf_freq_l[horizonindx,varindx];
      th_ = cs_freq_l;

      {GT_lesscritval, q_grid, n_nonbinding} = computeGT(q_grid);

      if GT_lesscritval == 1;
         /* starting point is included in CS */
         direction = -1; /* decrease lower bound */
      else;
         /* starting point is not included in CS */
         direction = 1; /* increase lower bound */
      endif;

      terminate = 0;

      do until terminate;
         
         th_ = cs_freq_l;

         {GT_lesscritval, q_grid, n_nonbinding} = computeGT(q_grid);

         /* Termination criterion
         */
         if direction == -1;
            /* we are decreasing the lower bound and find transition
            ** included -> not included OR the lower bound IRF >= 0
            */   
            if (GT_lesscritval == 0)
                OR ( (YYirfrestr[1]==1) and ((cs_freq_l-cs_step) < 0) );
                terminate = 1;
            endif;
         else; 
            /* we are increasing the lower bound and looking for transition
            ** not included -> included without violating IRF >=0
            */
            if (GT_lesscritval == 1) and (NOT ((YYirfrestr[1]==1) and (cs_freq_l < 0)));
               terminate = 1;
            endif;
         endif;
               
         cs_freq_l = cs_freq_l + direction*cs_step;

      endo;

      if direction == -1;
         /* we lowered the lower bound too much */
         varirf_freq_l[horizonindx,varindx] = cs_freq_l + cs_step;
         cs_freq_l = cs_freq_l + cs_step;
      else;
         /* we increased the lower bound */
         varirf_freq_l[horizonindx,varindx] = cs_freq_l;
      endif;
      "Lower Bound" cs_freq_l;
      pause(1e-2);

      /** set finish time **/
      tfin = date;
      "============================================";
      "Elapsed Time"; etstr(ethsec(tstart,tfin));

      if start_bayes == 1;
         "Start from Bayes";
      else;
         "Start from Freq ";
      endif;
      "Variable:   " varindx;
      "Horizon:    " horizonindx;
      "Start Intv: " varirf_l[horizonindx,varindx]~varirf_h[horizonindx,varindx];        
      "Freq Intv:  " varirf_freq_l[horizonindx,varindx]~varirf_freq_h[horizonindx,varindx];       
      "============================================";
      pause(1e-2);

      horizonindx = horizonindx + 1;

    endo;

    varindx = varindx + 1;

endo;

opath  = resupath $+ "\\freqrun" $+ freqrun;
odraws = opath $+ "\\" $+ "varirfs" $+ sel_str; 
chk_dir(opath);

create fhdraws = ^odraws with IRF, nvar_*nirf_, 8;
writer(fhdraws,( vec(varirf_freq_l)'|vec(varirf_freq_h)' ));
closeall;

output off;

end;

/****************************************************************/

proc (3) = computeGT(q_grid);
local GT_lesscritval, GT_val, i, q, q_dim, q_grid_new, endloop,
      n_nonbinding, critval_final;

GT_lesscritval = 0;
q_dim = rows(q_grid);
q_grid_new = q_grid;

"Testing " th_;
pause(1e-5);

i  = 1;
endloop = 0;
do until (i > q_dim) or (endloop == 1);

  q = q_grid[i,.]';
  {GT_val, critval_final, n_nonbinding} = GTobjconc(q);
  //i GT_val critval_final n_nonbinding (GT_val le critval_final); 

  if GT_val < critval_final;
     /* theta is included in CS */
     GT_lesscritval = 1;
     /* re-order the q-grid */
     if i == 1;
        q_grid_new = q_grid;
     elseif i < q_dim;
        q_grid_new = q_grid[i,.] | q_grid[1:i-1,.] | q_grid[i+1:q_dim,.];
     elseif i == q_dim;
        q_grid_new = q_grid[q_dim,.] | q_grid[1:q_dim-1,.];
     endif;
     endloop = 1;
  endif;
  i = i+1;

endo;

retp(GT_lesscritval,q_grid_new,n_nonbinding);
endp;

/****************************************************************/

proc(3) = GTobjconc(q);
/* uses only q as input - runs standard QP solver to find mu
** proc uses globals psi_hat_, YYirfrestr_, th_, nvar_
*/
local q_dim, th_dim, M_mu, M_th, Sq, 
      Sigma, Dsqrt, Dsqrt_inv, D1sqrt_inv, D2sqrt_inv, Omega, W, 
      phi_hat, phi2_hat, mu_hat, delta, fcn,
      nonbinding, n_nonbinding, 
      Omega_reduced, critval_final,
      fcn1;

q_dim  = nvar_;
th_dim = rows(th_);
M_mu   = zeros(th_dim,mu_dim_)|eye(mu_dim_);
M_th   = eye(th_dim)|zeros(mu_dim_,th_dim);
Sq     = eye(th_dim+mu_dim_).*.(q');

// Sigma and its factorization: Sigma = Dsqrt*Omega*Dsqrt
Sigma      = Sq*lambda_hat_*Sq';
Dsqrt      = eye(rows(Sigma)).*sqrt(diag(Sigma));
Dsqrt_inv  = inv(Dsqrt);
D1sqrt_inv = Dsqrt_inv[1:th_dim,1:th_dim];
D2sqrt_inv = Dsqrt_inv[th_dim+1:th_dim+mu_dim_,th_dim+1:th_dim+mu_dim_];
Omega     = Dsqrt_inv*Sigma*Dsqrt_inv;

// Weight Matrix
W = Dsqrt_inv*Dsqrt_inv;

// Determine mu-hat
phi_hat  = (Sq*psi_hat_).*(ones(th_dim,1)|YYirfrestr_[th_dim+1:th_dim+mu_dim_]) - M_th*th_;
phi2_hat = phi_hat[th_dim+1:th_dim+mu_dim_];
mu_hat   = phi2_hat .* (phi2_hat .ge 0);

// compute objective function
delta   = phi_hat - M_mu*mu_hat;
fcn     = delta'*W*delta;
//fcn1    = delta[1]^2*W[1,1];
//fcn fcn1;

// determine number of binding moment conditions
phi2_hat   = phi2_hat.*diag(D2sqrt_inv);
nonbinding = phi2_hat .> 1.96*ln(ln(nobs));
n_nonbinding = sumc(nonbinding);

// Simulated Critical Value
// remove rows and columns for non-binding moment conditions from Winv
nonbinding    = zeros(th_dim,1)|nonbinding;
Omega_reduced = delif(Omega,nonbinding);
Omega_reduced = delif(Omega_reduced',nonbinding)';
critval_final = critvalcalc(th_dim,tau_,Omega_reduced,critval_nsim_);

retp(fcn,critval_final,n_nonbinding);
endp;

/***************************************************************/

#include simulate_var.src
#include estimate_lambda.src
#include estimate_irfs.src
#include select_irfs.src
#include critvalcalc.src






