/******************************************************************/
/* filename :       vf_qcs_qgrid.g                                */
/* Description:                                                   */
/*                                                                */
/******************************************************************/

new;
closeall; 
cls;

#include vf_spec.g; 
#include pathspec.g;

// Define some constants 
q_dim          = nvar_;
lambda_nrep_   = 2000;

freqrun = "1" $+ "idset"; 
lambda_scale_ = 1;
cutoff = 1E-1;

/* import q_grid */
qgridpath = workpath $+ "\\" $+ prioname;
load path = ^qgridpath q_grid;

/******************************************************** 
* Import data series                                    *
********************************************************/

#include vf_loaddata.g;
k = 1+nvar_*nlags_;

/** start log file **/
outstr = resupath $+ "\\" $+ ftos(timedt,"%*.*lf",1,0) $+ "qidset" $+ freqrun $+ ".log"; 
output file = ^outstr reset;

q_dim = rows(q_grid);
q_cs  = zeros(1,nvar_);

/** set start time **/
tstart = date;   

/* estimate impulse responses
*/
"Computing Impulse Responses";
pause(1e-5);
{ YYirf, phi_hat, sigma_hat } = estimate_irfs(YY,XX,nirf_);
  YYirfselect                 = select_irfs(YYirf,signrestrictionsindx_);
psi_hat_   = vec(YYirfselect');
YYirfrestr_= signrestrictionsindx_[.,3];
mu_dim_    = rows(YYirfrestr_);

/* estimate lambda (covariance matrix of phi_hat)
** we are using a fixed seed for the bootstrap of phi_hat
*/
"Estimate Lambda via Bootstrap";
pause(1e-5);
lambda_hat_ = estimate_lambda(YY,XX,lambda_nrep_,signrestrictionsindx_);
lambda_hat_ = lambda_hat_/lambda_scale_;

/*************************************************************** 
* Iterate over q to determine which ones are included in IDset *
***************************************************************/
"Generate IDset Set for q";
q_ind  = 1;
do until q_ind > q_dim;

   q = q_grid[q_ind,.]';
   GT_val = GTobjconc(q);

   if GT_val le cutoff;
      // include q into IDset
      q_cs = q_cs | q';
   endif;

   if fmod(q_ind,q_dim/10)==0; ".";; pause(1e-20); endif;

   q_ind = q_ind + 1;
endo;

/** set finish time **/
tfin = date;
"";
"============================================";
"Elapsed Time"; etstr(ethsec(tstart,tfin));

q_cs = q_cs[2:rows(q_cs),.];

"Number of q's in IDset" rows(q_cs) " out of " rows(q_grid);

opath  = resupath $+ "\\freqrun" $+ freqrun;
odraws = opath $+ "\\" $+ "qidset"; 
chk_dir(opath);

create fhdraws = ^odraws with qCS, nvar_, 8;
writer(fhdraws,q_cs);
closeall;

output off;

end;

/****************************************************************/

proc(1) = GTobjconc(q);
/* uses only q as input - runs standard QP solver to find mu
** proc uses globals psi_hat_, YYirfrestr_, th_, nvar_
*/
local q_dim, Sq, 
      Sigma, Dsqrt, Dsqrt_inv, Omega, W, 
      phi_hat, mu_hat, delta, fcn, 
      nonbinding, n_nonbinding, 
      Omega_reduced, critval_final;

q_dim    = nvar_;
Sq       = eye(mu_dim_).*.(q');

// Sigma and its factorization: Sigma = Dsqrt*Omega*Dsqrt
Sigma = Sq*lambda_hat_*Sq';
Dsqrt = eye(rows(Sigma)).* sqrt(diag(Sigma));
Dsqrt_inv = inv(Dsqrt);
Omega = Dsqrt_inv*Sigma*Dsqrt_inv;

// Weight Matrix
W   = Dsqrt_inv*Dsqrt_inv;

// (Trivial) Solution to quadratic programming problem
phi_hat  = (Sq*psi_hat_).*YYirfrestr_;
mu_hat   = phi_hat .* ( phi_hat .ge 0 );

// compute objective function
delta   = phi_hat - mu_hat;
fcn     = delta'*W*delta;

retp(fcn);
endp;

/***************************************************************/

#include simulate_var.src
#include estimate_lambda.src
#include estimate_irfs.src
#include select_irfs.src
